<?php

declare(strict_types=1);

namespace App\Entity;

use App\Repository\DrinkTypeRepository;
use DateTimeImmutable;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

#[ORM\Entity(repositoryClass: DrinkTypeRepository::class)]
#[ORM\Table(name: 'drink_type')]
class DrinkType
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column(type: 'integer')]
    private null|int $id = null;

    #[ORM\Column(type: 'datetime_immutable')]
    private readonly DateTimeImmutable $createdAt;

    #[ORM\Column(type: 'datetime_immutable')]
    private DateTimeImmutable $updatedAt;

    #[ORM\OneToMany(mappedBy: 'drinkType', targetEntity: OrderItem::class)]
    private Collection $orderItems;

    #[ORM\Column(type: 'string', length: 255, unique: true)]
    private string $name;
    #[ORM\Column(type: 'text', nullable: true)]
    private null|string $description = null;
    #[ORM\Column(type: 'integer')]
    private int $wantedStock = 10;

    #[ORM\Column(type: 'integer')]
    private int $currentStock = 0;


    public function __construct(
    ) {
        $this->createdAt = new DateTimeImmutable();
        $this->updatedAt = new DateTimeImmutable();
        $this->orderItems = new ArrayCollection();
    }

    public function getCurrentStock(): int
    {
        return $this->currentStock;
    }

    public function setCurrentStock(int $currentStock): void
    {
        $this->currentStock = $currentStock;
    }

    public function getId(): null|int
    {
        return $this->id;
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;
        return $this;
    }

    public function getDescription(): null|string
    {
        return $this->description;
    }

    public function setDescription(null|string $description): self
    {
        $this->description = $description;
        return $this;
    }

    public function getWantedStock(): int
    {
        return $this->wantedStock;
    }

    public function setWantedStock(int $wantedStock): self
    {
        $this->wantedStock = $wantedStock;
        return $this;
    }


    public function getCreatedAt(): DateTimeImmutable
    {
        return $this->createdAt;
    }

    public function getUpdatedAt(): DateTimeImmutable
    {
        return $this->updatedAt;
    }

    public function getOrderItems(): Collection
    {
        return $this->orderItems;
    }

    #[ORM\PrePersist]
    private function updateTimestamp(): void
    {
        $this->updatedAt = new DateTimeImmutable();
    }
}
